@@ -34,4 +34,4 @@ environment = environment_factory(regeneration_factor=0.1, clazz=Environment3d)
 stop_condition = lambda: StepLimitStopCondition(90)
 
 address_provider = address.SequenceAddressProvider
-stats = lambda: MultipleStatistics([CsvStatistics(), PsiStatistics()])
\ No newline at end of file
+stats = SimpleStatistics
\ No newline at end of file
